# Backtrace SAML Service

## Getting started

### System requirements

NodeJS `v10.15.0` or later

### Installing

```
$ apt-get install backtrace-saml
```

This will install a package located at `/opt/backtrace/services/backtrace-saml/`.

### Configuration

In order to get started with your saml service a configuration file needs to be set up. A default configuration file is included in the package's root directory `/saml.conf` with a defaulted configuration of:

```
{
  "bind": {
    "database": {
      "client": "sqlite3",
      "connection": {
        "filename": "/etc/backtrace/saml/saml.production.db"
      }
    },
    "port": 6446
  }
}
```

However, the preferred location the service's configuration is `/etc/backtrace/saml/saml.conf` in line with **Backtrace I/O** service axioms.

#### SSL settings

**NOTE** - on-premise service installs with colocated services should run over `http` and set to use proxy. If that is the case, then this section may be ignored.

To run your service over https you need to complete the following settings within the `saml.conf`'s `bind` object.

Fields:

`bind.ssl` - _Object_ - contains location for servers ssl certificate and key

`bind.ssl.certificate` - _String_ - absolute path of ssl cert

`bind.ssl.key` - _String_ - absolute path of ssl key

`bind.ssl.certificate_chain_file` - _String_ - absolute path of ssl certificate chain file (only needed for use of self-signed certificate)

`bind.port` - _Int_ - server defaults to use port 6446 in line with our service layer specification. If a custom port is requested then one may be declared here.

```
{
  ...
  "bind": {
    ...
    "ssl": {
      "key": "/etc/letsencrypt/saml/privkey.pem",
      "certificate": "/etc/letsencrypt/saml/fullchain.pem"
    },
    "port": 6446
  }
}
```

#### Database settings

Current settings found in the default `saml.conf` are the recommended settings for on premise use of the SAML service. If you have any questions concers about these settings, please first consult Backtrace support at support@backtrace.io.

#### Optional Configuration settings

Fields:

`enableBacktraceReporting` - _Boolean_ - If truthy, enables reporting to be sent to Backtrace for errors that occur in the backtrace-saml-service. Information that is sent to Backtrace includes: Error object, hostname, backtrace-saml-service version, and Email (if available, email associated with user will be sent).

`samlOptions` - _Object_ - this package utilizes [passport.js](http://www.passportjs.org/) and [passport-saml](https://github.com/node-saml/passport-saml)'s authentication strategy. The `samlOptions` object alows you to override (and add to) the SAML strategies initialization settings. Current config options used by the service are:

```
{ disableRequestedAuthnContext: true }
```

> `disableRequestedAuthnContext: true` disables backtrace-saml-service from requiring the IdP to supply an authentication context in the SAML response as not every IdP will respond with one.

example:

```
{
  ...
  "samlOptions": {
    "disableRequestedAuthnContext": false,
    "authnContext": ["urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport"]
  }
}
```

**NOTE** - as of version `1.3.18`, the default ACS url constructed by the `backtrace-saml` will be of the form:

```
// This ACS url tells the IdP to send SAML responses through
// The service layer proxy
https://<hostname>/api/sso/saml/<hostname>
```

where _some_ legacy systems may have already defined an ACS url with the shape of:

```
// This ACS url tells the IdP to send SAML response straight
// to the backtrace-saml server
https://<hostname>:<backtrace-saml port>/saml/<hostname>
```

In order to reconcile the differences in ACS url shapes, the following is added to the `saml.conf`:

```
{
  ...
  "samlOptions": {
    "pathPrefix": "/saml"
  }
}
```

## Operation

Using `systemctl` you can start, stop, restart, or find status of the service

| description        | command                            |
| ------------------ | ---------------------------------- |
| starting service   | `$ service backtrace-saml start`   |
| stopping service   | `$ service backtrace-saml stop`    |
| restarting service | `$ service backtrace-saml restart` |
| status of service  | `$ service backtrace-saml restart` |

## Upgrading to a new version

You can easily upgrade to a new version of backtrace-saml as we continue making updates for bug fixes / new features.

```
$ apt-get update
$ apt-get install backtrace-saml
$ systemctl daemon-reload
$ service backtrace-saml restart
```

## Downgrading to a previous version

In the event that you may need to rollback `backtrace-saml` you can easily do so by installing a targeted version and then restarting the service.

```
$ apt-get install backtrace-saml=<targeted version>
$ systemctl daemon-reload
$ service backtrace-saml restart
```
